#ifndef _FIELDSAT_H

#include <stdio.h>

#define F_CPU 16000000UL

#define FIELDSAT_SAMPLE_SIZE 32

/*
#define OLD_FORMAT
*/

#define CONFIG_FILE "config.txt"

enum sample_type {
	SAMPLE_TYPE_GO,
	SAMPLE_TYPE_S1,
	SAMPLE_TYPE_S2,
	SAMPLE_TYPE_SD
};

struct fieldsat_s1 {
	double acc_low[3];
	double ang_vel[3];
	double solar_luminosity;
	double solar_voltage;
};

struct fieldsat_s2 {
	double pressure;
	double temp_int;
	double temp_ext;
};

struct fieldsat_boolean {
	unsigned int value;
};

struct fieldsat_sample {
	unsigned long timestamp;
	int type;
	struct fieldsat_s1 s1;
	struct fieldsat_s2 s2;
	struct fieldsat_boolean boolean;
};

struct fieldsat_state {
	double position[3];
	double velocity[3];
	double acceleration[3];
	double avg_acc[3];
	/* A quaternion. */
	double rotation[4];
	double ang_vel[3];
	double pressure;
	double altitude;
	double temp_int;
	double temp_ext;
	double solar_luminosity;
	double solar_voltage;
	unsigned int solar_is_deployed;
};

FILE *stream_open(int argc, char *argv[]);

void load_config(const char *filename);

int synchronize(FILE *fp);

int read_sample(FILE *fp, struct fieldsat_sample *sample);

void sample_init(struct fieldsat_sample *sample);

void sample_print(const struct fieldsat_sample *sample);

void state_update(struct fieldsat_state *state,
	const struct fieldsat_sample *sample, double t);

void state_init(struct fieldsat_state *state);

void state_print(const struct fieldsat_state *state);

extern double ACC_LOW_ZERO[3];
extern double ACC_LOW_VARIANCE[3];
extern double ANG_VEL_ZERO[3];
extern double ANG_VEL_VARIANCE[3];

#endif
