#include <stdio.h>
#include <stdlib.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "SDL.h"
#include "SDL_image.h"

#include "fieldsat.h"

#define SCREEN_WIDTH 1000
#define SCREEN_HEIGHT 800

#define NUM_SAMPLES 16384

#define TIME_WIDTH 3.0

/*
#define DISPLAY_TYPE GL_POINTS
*/
#define DISPLAY_TYPE GL_LINE_STRIP

static const double LINE_COLOR[3] = { 0.0, 0.4, 0.0 };
static const double POINT_COLOR[3] = { 0.0, 0.0, 0.0 };
static const double ARROW_COLOR[3] = { 1.0, 0.0, 0.0 };
static const double REF_COLOR[3] = { 0.7, 0.7, 1.0 };

static struct fieldsat_sample samples[NUM_SAMPLES];
static double times[NUM_SAMPLES];

static unsigned int current_sample = 0;
static double acc_x, acc_y;
static unsigned long previous_timestamp = 0;
static double current_sample_time = 0.0;
static double current_time = 0.0;
static double last_second = 0.0;
static unsigned long sample_count = 0;

static FILE *fp;

static void init(void)
{
	/*
	glEnable(GL_LINE_SMOOTH);
	glHint(GL_LINE_SMOOTH_HINT, GL_FASTEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	*/
	glClearColor(0.9, 0.9, 0.9, 0.0);
	/*
	glPointSize(20.0);
	*/

	synchronize(fp);
}

static void update(void)
{
	struct fieldsat_sample *sample;
	double t;

	if (current_time - last_second > 1.0) {
		printf("%f samples / second\n", sample_count / (current_time - last_second));
		last_second = current_time;
		sample_count = 0;
	}
	/*
	*/

	if (current_sample_time > current_time + 0.05 * TIME_WIDTH)
		return;

	do {
		sample = &samples[current_sample];

		if (!read_sample(fp, sample)) {
			synchronize(fp);
			if (!read_sample(fp, sample)) {
				fprintf(stderr, "Sample read error.\n");
				exit(1);
			}
		}

		if (sample->type != SAMPLE_TYPE_S1)
			return;

		sample_count++;

		sample->s1.acc_low[0] = (sample->s1.acc_low[0] - 512.0) / 512.0;
		sample->s1.acc_low[1] = (sample->s1.acc_low[1] - 512.0) / 512.0;
		sample->s1.acc_low[2] = (sample->s1.acc_low[2] - 512.0) / 512.0;
		sample->s1.ang_vel[0] = (sample->s1.ang_vel[0] - 512.0) / 512.0;
		sample->s1.ang_vel[1] = (sample->s1.ang_vel[1] - 512.0) / 512.0;
		sample->s1.ang_vel[2] = (sample->s1.ang_vel[2] - 512.0) / 512.0;
		sample->s1.solar_luminosity = (sample->s1.solar_luminosity - 512.0) / 512.0;
		sample->s1.solar_voltage = (sample->s1.solar_voltage - 512.0) / 512.0;
		times[current_sample] = current_sample_time;

		if (current_sample_time == 0.0)
			t = current_time;
		else
			t = (double) (sample->timestamp - previous_timestamp) / F_CPU;
		current_sample_time += t;

		/*
		sample_print(sample);
		*/

		if (t > 1.0)
			printf("Warning, t = %f.\n", t);

		if (sample->timestamp < previous_timestamp)
			printf("Wrap (%lu - %lu) = %lu\n", sample->timestamp, previous_timestamp, sample->timestamp - previous_timestamp);

		previous_timestamp = sample->timestamp;

		current_sample = (current_sample + 1) % NUM_SAMPLES;
	} while (current_sample_time < current_time - 0.05 * TIME_WIDTH);
}

static void display(void)
{
	unsigned int i, j;

	glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0.0, 1.0, 0.0, 1.0, 1.0, 100.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslated(0.0, 0.1, -5.0);
	glScaled(1.0, 0.8, 1.0);

	glColor3dv(REF_COLOR);
	glBegin(GL_LINES);
	for (j = 0; j < 9; j++) {
		glVertex3d(0.0, (j + 0.5) / 9.0, 0.0);
		glVertex3d(1.0, (j + 0.5) / 9.0, 0.0);
	}
	glEnd();

	/*
	glColor3dv(POINT_COLOR);
	for (j = 0; j < 3; j++) {
		glBegin(GL_POINTS);
		for (i = 0; i < NUM_SAMPLES - 1; i++) {
			double lambda, time;
			struct fieldsat_sample *sample;

			sample = &samples[(current_sample + i + 1) % NUM_SAMPLES];
			time = times[(current_sample + i + 1) % NUM_SAMPLES];

			if (time < current_time - TIME_WIDTH)
				continue;
			lambda = (time - (current_time - TIME_WIDTH)) / TIME_WIDTH;
			glVertex3d(lambda,
				(j + 0.5) / 9.0 + sample->acc_low[j] / 2.0,
				0.0);
		}
		glEnd();
	}
	*/
	glColor3dv(LINE_COLOR);
	/*
	*/
	for (j = 0; j < 3; j++) {
		glBegin(DISPLAY_TYPE);
		for (i = 0; i < NUM_SAMPLES - 1; i++) {
			double lambda, time;
			struct fieldsat_sample *sample;

			sample = &samples[(current_sample + i + 1) % NUM_SAMPLES];
			time = times[(current_sample + i + 1) % NUM_SAMPLES];

			if (time < current_time - TIME_WIDTH)
				continue;
			lambda = (time - (current_time - TIME_WIDTH)) / TIME_WIDTH;
			glVertex3d(lambda,
				(j + 0.5) / 9.0 + sample->s1.acc_low[j] / 2.0,
				0.0);
		}
		glEnd();
	}
	/*
	*/
	for (j = 0; j < 3; j++) {
		glBegin(DISPLAY_TYPE);
		for (i = 0; i < NUM_SAMPLES - 1; i++) {
			double lambda, time;
			struct fieldsat_sample *sample;

			sample = &samples[(current_sample + i + 1) % NUM_SAMPLES];
			time = times[(current_sample + i + 1) % NUM_SAMPLES];

			if (time < current_time - TIME_WIDTH)
				continue;
			lambda = (time - (current_time - TIME_WIDTH)) / TIME_WIDTH;
			glVertex3d(lambda,
				(j + 3 + 0.5) / 9.0 + sample->s1.ang_vel[j] / 2.0,
				0.0);
		}
		glEnd();
	}
	glBegin(DISPLAY_TYPE);
	for (i = 0; i < NUM_SAMPLES - 1; i++) {
		double lambda, time;
		struct fieldsat_sample *sample;

		sample = &samples[(current_sample + i + 1) % NUM_SAMPLES];
		time = times[(current_sample + i + 1) % NUM_SAMPLES];

		if (time < current_time - TIME_WIDTH)
			continue;
		lambda = (time - (current_time - TIME_WIDTH)) / TIME_WIDTH;
		glVertex3d(lambda,
			(6 + 0.5) / 9.0 + sample->s1.solar_luminosity / 10.0,
			0.0);
	}
	glEnd();
	glBegin(DISPLAY_TYPE);
	for (i = 0; i < NUM_SAMPLES - 1; i++) {
		double lambda, time;
		struct fieldsat_sample *sample;

		sample = &samples[(current_sample + i + 1) % NUM_SAMPLES];
		time = times[(current_sample + i + 1) % NUM_SAMPLES];

		if (time < current_time - TIME_WIDTH)
			continue;
		lambda = (time - (current_time - TIME_WIDTH)) / TIME_WIDTH;
		glVertex3d(lambda,
			(7 + 0.5) / 9.0 + sample->s1.solar_voltage / 10.0,
			0.0);
	}
	glEnd();

	glColor3dv(ARROW_COLOR);
	glBegin(GL_LINES);
	glVertex3d(0.5, 0.5, 0.0);
	glVertex3d(0.5 + acc_x * 5.0, 0.5 + acc_y * 5.0, 0.0);
	glEnd();
}

#ifdef main
#undef main
#endif

int main(int argc, char *argv[])
{
	SDL_Surface *screen;
	SDL_Event event;
	Uint32 ticks, ticks_delta;
	unsigned int quit = 0;

	fp = stream_open(argc, argv);

	if (SDL_Init(SDL_INIT_VIDEO) == -1) {
		fprintf(stderr, "Can't initialize SDL: %s\n", SDL_GetError());
		return 1;
	}
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
	screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 32,
		SDL_ANYFORMAT | SDL_OPENGL );
	if (screen == NULL) {
		fprintf(stderr, "Can't set video mode: %s\n", SDL_GetError());
		return 1;
	}

	init();

	ticks = SDL_GetTicks();
	while (!quit) {
		ticks_delta = SDL_GetTicks() - ticks;
		ticks += ticks_delta;
		current_time += ticks_delta / 1000.0;
		update();
		display();
		SDL_GL_SwapBuffers();
		while (SDL_PollEvent(&event)) {
			if (event.type == SDL_KEYDOWN)
				quit = 1;
			else if (event.type == SDL_QUIT)
				quit = 1;
		}
	}

	SDL_FreeSurface(screen);
	SDL_Quit();

	return 0;
}
